@@ -1,5 +1,5 @@
 
-__version__ = "0.0.7"
+__version__ = "0.0.8"
 from estime2.config import (
     get_option,
     option_context,
@@ -8,5 +8,11 @@ from estime2.config import (
     set_option
 )
 from estime2.age import Age
-from estime2.helper import compute_sd
+from estime2.metric import (
+    get_agediff_range,
+    get_agediff_sparsity,
+    get_correction_magni,
+    get_correction_sd,
+    get_num_cells
+)
 from estime2.provpoptable import ProvPopTable
