@@ -11,6 +11,7 @@ RegisteredOption = namedtuple(
 )
 age_down_to = 40
 age_max = 99
+age_prop_size = 5
 comp_neg_temp_out = 'TEM'
 comp_neg_emi = 'EMI'
 comp_neg_npr_out = 'NPR, 2018-07-01'
@@ -24,7 +25,7 @@ comp_pos_immi = 'IMM'
 comp_pos_interprov_in = 'IIM'
 comp_pos_intraprov_in = 'RAI'
 comp_end = [comp_pos_npr_in]
-method_all = ['1dist', 'prop']
+method_all = ['1dist', 'filler', 'prop']
 method = '1dist'
 pop_age = 'Age'
 pop_birth = 'BTH'
@@ -108,13 +109,20 @@ def is_list_str(x, opname, n = None):
             raise ValueError(msg3)
     return None
 
-def is_nonnegative(x):
+def is_nonnegative(x, opname, force_positive = False):
     if not isinstance(x, int):
-        msg1 = "pop.at_least must be an integer, not of {}."
-        raise TypeError(msg1.format(type(x)))
-    if x < 0:
-        msg2 = "pop.at_least must be a nonnegative integer, not {}."
-        raise ValueError(msg2.format(x))
+        raise TypeError(
+            f"{opname} must be an integer, not of {type(x)}."
+        )
+    x_test = None
+    if force_positive:
+        x_test = (x <= 0)
+    else:
+        x_test = (x < 0)
+    if x_test:
+        raise ValueError(
+            f"{opname} must be a nonnegative integer, not {x}."
+        )
     return None
 
 def is_list_containing(x, opname):
@@ -149,6 +157,8 @@ def is_method(x):
 
 is_int_age_max = lambda x: is_int(x, 'age.max')
 is_int_age_down_to = lambda x: is_int(x, 'age.down_to')
+is_pos_age_prop_size = lambda x: is_nonnegative(x, 'age.prop_size', True)
+is_pos_pop_at_least = lambda x: is_nonnegative(x, 'pop.at_least')
 is_str_comp_neg_temp_out = lambda x: is_str(x, 'comp_neg.temp_out')
 is_str_comp_neg_emi = lambda x: is_str(x, 'comp_neg.emi')
 is_str_comp_neg_npr_out = lambda x: is_str(x, 'comp_neg.npr_out')
@@ -173,7 +183,8 @@ is_list_containing_comp_end = lambda x: is_list_containing(x, 'comp.end')
 _global_config = {
     'age': {
         'down_to': age_down_to,
-        'max': age_max
+        'max': age_max,
+        'prop_size': age_prop_size
     },
     'comp': {
         'end': comp_end
@@ -228,6 +239,16 @@ _registered_options = {
             'greater than or equal to 100 is classified as `100+`.', 
         validator = is_int_age_max
     ),
+    'age.prop_size': RegisteredOption(
+        key = 'age.prop_size',
+        defval = age_prop_size,
+        doc =\
+            "\n (int)\n    " +\
+            'A size of period to use when applying the ' +\
+            'proportional method. It must be a positive integer.' +\
+            'Default is 5.',
+        validator = is_pos_age_prop_size
+    ),
     'comp.end': RegisteredOption(
         key = 'comp.end',
         defval = comp_end,
@@ -351,7 +372,8 @@ _registered_options = {
         doc =\
             "\n (str)\n    " +\
             "A method to apply counter-adjustments to a population " +\
-            "table. Must be either one of '1dist' or 'prop'.",
+            "table. Must be either one of '1dist', 'filler', " +\
+            "or 'prop'.",
         validator = is_method
     ),
     'pop.age': RegisteredOption(
@@ -380,7 +402,7 @@ _registered_options = {
             "A minimum end-of-period population estimate that needs\n" +\
             "to be kept." +\
             "This option cannot be None.",
-        validator = is_nonnegative
+        validator = is_pos_pop_at_least
     ),
     'pop.end': RegisteredOption(
         key = "pop.end",
