@@ -842,6 +842,17 @@ class ProvPopTable(pd.DataFrame):
 
         return result
 
+    def get_pop_end(self):
+        '''
+        Get self.__pop_end, the name of the end-of-period population.
+
+        Usage
+        -----
+        `ProvPopTable.get_pop_end()`
+        '''
+
+        return self.__pop_end
+
     def get_pop_groups(self):
         '''
         Get self.__pop_sex and self.__pop_age, the names of sex and age
@@ -1022,9 +1033,9 @@ class ProvPopTable(pd.DataFrame):
         Arguments
         ---------
         * `comp`: a str; the same argument as `self.get_J(comp)`.
-        * `method`: a str; either '1dist' or 'prop'. If `None`, then the
-            global option value `estime2.options.method` is used. See
-            Details.
+        * `method`: a str; either '1dist', 'filler', or 'prop'. If `None`, 
+            then the global option value `estime2.options.method` is used. 
+            See Details below.
         * `J`: a dict of "str: pd.DataFrame"; a return value of 
             `self.get_J(comp)`. If `None`, then `self.get_J(comp)` is
             evaluated.
@@ -1035,29 +1046,46 @@ class ProvPopTable(pd.DataFrame):
         the actual amount to be either added or subtracted to the `comp`
         values of `self`. Depending on the `method` argument, the way in
         which those corrections are calculated gets different.
+
         Suppose that neighbouring ages are from 56 to 95 and the total 
         value to be modified is 4. Suppose also that, from the result of 
         `self.get_J(comp)`, the maximum counter-adjustable value for ages 
         91, 92, 93, 94, and 95 are computed as 1, 1, 2, 1, and 0 
         respectively.
-        If `method = '1dist'`, then `self.get_K()` distributes 1 to the
-        counter-adjustable component values starting from the maximum
-        counter-adjustable age so that the modifiable component value is 
-        balanced out. For the above scenario, '1dist' starts to put 1 to 
-        the `comp` of age 94 since the `comp` at age 95 is not correctable,
-        1 to age 93, 1 to age 92, and finally 1 to age 91.
-        If `method = 'prop'`, then `self.get_K()` distributes the maximum
-        correctable amount starting from the maximum counter-adjustable
-        age so that the modifiable component value is balanced out. For the
-        above scenario, it puts 1 to the age 94, 2 to the age 93 since that
-        is the maximum correctable amount, and finally 1 to the age 92.
+
+        1. If `method = '1dist'`, then `self.get_K()` distributes 1 to the
+            counter-adjustable component values starting from the maximum
+            counter-adjustable age so that the modifiable component value 
+            is balanced out. For the above scenario, '1dist' starts to put 
+            1 to the `comp` of age 94 since the `comp` at age 95 is not 
+            correctable, 1 to age 93, 1 to age 92, and finally 1 to age 91.
+        2. If `method = 'filler'`, then `self.get_K()` distributes the 
+            maximum correctable amount starting from the maximum 
+            counter-adjustable age so that the modifiable component value 
+            is balanced out. For the above scenario, it puts 1 to the age 
+            94, 2 to the age 93 since that is the maximum correctable 
+            amount, and finally 1 to the age 92.
+        3. If `method = 'prop'`, then `self.get_K()` computes the 
+            proportions of end-of-period populations in a given period,
+            usually in a length of 5 years (which can be changed via 
+            `estime2.options.age.prop_size`), compare them with the maximum
+            correctable values at each record's component, and take the
+            minimum of the two. In each counter-adjustment, this method
+            checks whether the entire value of modification is distributed
+            within the component of 5-year period. If yes, terminate the
+            process; otherwise, the remaining modification value is
+            distributed to the record of the highest proportions as much as
+            possible. In case of tie proportions, take the oldest
+            neighbouring-age record within the period. The process 
+            continues until all the modification value is distributed.
         '''
 
         method = return_goption_if_None('method', method)
         prev_method = get_option('method')
         set_option('method', method)
         set_option('method', prev_method)
-
+        
+        comp_in_comp_end = comp in self.__comp_end
         pop_groups = [self.__pop_sex, self.__pop_age]
         comp_J = "{0}_J".format(comp)
         dfs_comp = None
@@ -1080,7 +1108,7 @@ class ProvPopTable(pd.DataFrame):
                     to_modify_val_total -= 1
                 else:
                     result.append(0)
-        else: # i.e. prop
+        elif method == 'filler':
             for index, row in df_comp_counter_adjust_reversed.iterrows():
                 if row[comp_J] > 0 and to_modify_val_total != 0:
                     min_comp_J_val = min(row[comp_J], to_modify_val_total)
@@ -1088,6 +1116,66 @@ class ProvPopTable(pd.DataFrame):
                     to_modify_val_total -= min_comp_J_val
                 else:
                     result.append(0)
+        elif method == 'prop':
+            problematic_sex = df_comp_modifiable[self.__pop_sex].values[0]
+            prop_max_age = df_comp_counter_adjust_reversed\
+                .iloc[0, :]\
+                [self.__pop_age]
+            prop_min_age = df_comp_counter_adjust_reversed\
+                .iloc[-1, :]\
+                [self.__pop_age]
+            if not comp_in_comp_end:
+                prop_max_age += 1
+                prop_min_age += 1
+            calculated_pop = self.calculate_pop()
+            pop_end_to_compare_reversed = calculated_pop.copy()\
+                .loc[lambda df: df[self.__pop_sex] == problematic_sex]\
+                .loc[lambda df: df[self.__pop_age] <= prop_max_age]\
+                .loc[lambda df: df[self.__pop_age] >= prop_min_age]\
+                .sort_values(pop_groups, ascending = [True, False])
+            
+            prop_size = get_option('age.prop_size')
+            pop_end_for_prop =\
+                pop_end_to_compare_reversed.iloc[:, -1].values
+            max_correctables =\
+                df_comp_counter_adjust_reversed.iloc[:, -1].values
+            pop_end_len = len(pop_end_for_prop)
+            # Note: len(pop_end_for_prop) == len(max_correctables)
+            for i in np.arange(pop_end_len, step = prop_size):
+                # min(...) is required to define `period` in case where 
+                # pop_end_len is not a multiple of prop_size
+                period = [i, min(i + prop_size, pop_end_len)]
+                props_numer = pop_end_for_prop[period[0]:period[1]]
+                props_denom = sum(props_numer)
+                props = props_numer / props_denom
+                props_x_m = np.array(
+                    list(map(int, np.round(to_modify_val_total * props)))
+                )
+                max_correct_for_each =\
+                    max_correctables[period[0]:period[1]]
+                min_from_each = np.minimum(props_x_m, max_correct_for_each)
+                # The last `[0]` in `loc_max_prop` is an application of
+                # the principle to take the oldest neighbouring-age record 
+                # within the period in case where the record of highest 
+                # proportion is not unique.
+                loc_max_prop = np.where(props == max(props))[0][0]
+                sum_min_from_each = sum(min_from_each)
+                to_modify_val_total -= sum_min_from_each
+                plmp = props_x_m[loc_max_prop]
+                mclmp = max_correct_for_each[loc_max_prop]
+                if plmp <= mclmp:
+                    more_possible_correction = mclmp - plmp
+                    min_mod_cor = min(
+                        to_modify_val_total,
+                        more_possible_correction
+                    )
+                    min_from_each[loc_max_prop] += min_mod_cor
+                    to_modify_val_total -= min_mod_cor
+                    result.extend(min_from_each)
+                else:
+                    result.extend(min_from_each)
+        else:
+            raise NotImplementedError
 
         df_comp_counter_adjust_reversed[comp_K] = result
         del df_comp_counter_adjust_reversed[comp_J]
@@ -1130,14 +1218,24 @@ class ProvPopTable(pd.DataFrame):
 
         comp_neg = self.get_comp_neg()
         comp_in_neg = comp in comp_neg
-        comp_J = '{0}_J'.format(comp)
-        comp_K = '{0}_K'.format(comp)
-        comp_L = '{0}_L'.format(comp)
+        comp_J = f'{comp}_J'
+        comp_K = f'{comp}_K'
+        comp_L = f'{comp}_L'
         
         J_all = self.get_J(comp)
         K = self.get_K(comp, method, J_all)
         J = J_all['records.to_modify']
-        
+
+        sum_mod_val = J[comp_J].sum()
+        p_sex = J[self.__pop_sex].values[0]
+        p_age = J[self.__pop_age].values
+        sum_counter_adjust = K[comp_K].sum()
+        assert sum_mod_val == sum_counter_adjust, \
+            f"The total modification value, {sum_mod_val}, " +\
+            f"at sex {p_sex} age(s) {p_age} and " +\
+            f"component {comp} is not equal to the sum of " +\
+            f"counter-adjustments, {sum_counter_adjust}."
+
         if comp_in_neg:
             J[comp_L] = -J[comp_J]
             del J[comp_J]
@@ -1280,17 +1378,39 @@ def calculate_counter_adjustable_in_comp(
     if len(comp_values) != len(pop_end_values):
         # This will never happen since to_counter_age_min is always 
         # at least -1.
-        raise NotImplementedError 
-    for val in zip(pop_end_values, comp_values):
-        if val[0] <= 0:
+        raise NotImplementedError
+    for pop_end_val, comp_val in zip(pop_end_values, comp_values):
+        if pop_end_val <= 0:
+            # If "end-of-period pop - 1" <= 0 (where 1 == pop.at_least),
+            # then that neighbouring age record can't absorb any correction
             to_append = 0
         elif comp_in_neg:
-            to_append = val[0]
+            # If "end-of-period pop - 1" > 0, i.e. it is correctable,
+            # then see if the component is negative.
+            # If it is, then the maximum correction (i.e. increase in the
+            # component value of the record, like increasing the death from
+            # 2 to 3) of this record must be the "end-of-period pop of that 
+            # record - 1" because if the negative component increases more
+            # than that, then the "end-of-period population - 1" of the 
+            # neighbouring age record will go below 1.
+            to_append = pop_end_val
         else:
-            to_append = val[1]
+            # If "end-of-period pop - 1" > 0, i.e. it is correctable,
+            # and the component is positive, then the maximum correction 
+            # (i.e. "decrease" in the component value of the record, like
+            # decreasing the immigration from 5 to 4) of this record must
+            # be the minimum of "end-of-period pop - 1" and the component 
+            # value. This is because, in the positive component, the 
+            # component value has to decrease (not increase) in 
+            # neighbouring ages in order to balance out the
+            # modification at the problematic record. If the 
+            # counter-adjustment goes below this minimum value, then either
+            # the end-of-period population or the component value will 
+            # become negative.
+            to_append = min(pop_end_val, comp_values)
         counter_adjustable.append(to_append)
 
-    df_comp_counter_adjust["{0}_J".format(comp)] = counter_adjustable
+    df_comp_counter_adjust[f"{comp}_J"] = counter_adjustable
     del df_comp_counter_adjust[comp]
 
     return df_comp_counter_adjust
