@@ -20,13 +20,16 @@ import os
 import pandas as pd
 pd.options.display.max_rows = 15
 
-
-def run():
+def call_tbl():
     files_above = os.listdir('..')
     starts_with_G = list(map(lambda x: x[0] == 'G', files_above))
     tbl_name = np.array(files_above)[starts_with_G][0]
     tbl = pd.read_excel('../' + tbl_name, sheet_name = 1)
     poptbl = ProvPopTable(tbl)
+    return poptbl
+
+def run():
+    poptbl = call_tbl()
 
     pop_age = 'Age'
     pop_end = 'Postcensal Population'
@@ -403,11 +406,7 @@ def run2():
 
     method = get_option('method')
 
-    files_above = os.listdir('..')
-    starts_with_G = list(map(lambda x: x[0] == 'G', files_above))
-    tbl_name = np.array(files_above)[starts_with_G][0]
-    tbl = pd.read_excel('../' + tbl_name, sheet_name = 1)
-    self = ProvPopTable(tbl)
+    self = call_tbl()
 
     comp_neg_to_use = self.get_comp_neg()
     comp_pos_to_use = self.get_comp_pos()
@@ -499,11 +498,7 @@ def run2():
     print(self_copy)
 
 def run3():
-    files_above = os.listdir('..')
-    starts_with_G = list(map(lambda x: x[0] == 'G', files_above))
-    tbl_name = np.array(files_above)[starts_with_G][0]
-    tbl = pd.read_excel('../' + tbl_name, sheet_name = 1)
-    poptbl = ProvPopTable(tbl)
+    poptbl = call_tbl()
     print(poptbl)
     print(poptbl.calculate_pop())
 
@@ -516,3 +511,142 @@ def run3():
         print(result[0].calculate_pop())
     
     return result
+
+def run4():
+
+    self = call_tbl()
+    comp = 'DTH'
+    comp_in_comp_end = False
+    method = 'prop'
+    pop_sex = 'Sex'
+    pop_age = 'Age'
+    pop_end = 'Postcensal Population'
+    problematic_sex = 1
+
+    pop_groups = self.get_pop_groups()
+    comp_J = f"{comp}_J"
+    dfs_comp = self.get_J(comp)
+    df_comp_modifiable = dfs_comp['records.to_modify']
+    df_comp_counter_adjust = dfs_comp['records.to_counter']
+    df_comp_counter_adjust_reversed = df_comp_counter_adjust\
+        .sort_values(pop_groups, ascending = [True, False])
+    to_modify_val_total = df_comp_modifiable[comp_J].sum()
+
+    print("df_comp_modifiable:")
+    print(df_comp_modifiable)
+    print("df_comp_counter_adjust_reversed:")
+    print(df_comp_counter_adjust_reversed)
+
+    comp_K = f"{comp}_K"
+    result = []
+    if method == '1dist':
+        for index, row in df_comp_counter_adjust_reversed.iterrows():
+            if row[comp_J] > 0 and to_modify_val_total != 0:
+                result.append(1)
+                to_modify_val_total -= 1
+            else:
+                result.append(0)
+    elif method == 'filler':
+        for index, row in df_comp_counter_adjust_reversed.iterrows():
+            if row[comp_J] > 0 and to_modify_val_total != 0:
+                min_comp_J_val = min(row[comp_J], to_modify_val_total)
+                result.append(min_comp_J_val)
+                to_modify_val_total -= min_comp_J_val
+            else:
+                result.append(0)
+    else: # proportional method
+        problematic_sex = df_comp_modifiable[pop_sex].values[0]
+        prop_max_age = df_comp_counter_adjust_reversed\
+            .iloc[0, :]\
+            [pop_age]
+        prop_min_age = df_comp_counter_adjust_reversed\
+            .iloc[-1, :]\
+            [pop_age]
+        if not comp_in_comp_end:
+            prop_max_age += 1
+            prop_min_age += 1
+        calculated_pop = self.calculate_pop()
+        pop_end_to_compare_reversed = calculated_pop.copy()\
+            .loc[lambda df: df[pop_sex] == problematic_sex]\
+            .loc[lambda df: df[pop_age] <= prop_max_age]\
+            .loc[lambda df: df[pop_age] >= prop_min_age]\
+            .sort_values(pop_groups, ascending = [True, False])
+        print("pop_end_to_compare_reversed:")
+        print(pop_end_to_compare_reversed)
+
+        prop_size = get_option('age.prop_size')
+        pop_end_for_prop =\
+            pop_end_to_compare_reversed.iloc[:, -1].values
+        if comp_in_comp_end:
+            pop_end_for_prop[0] = 0
+        max_correctables =\
+            df_comp_counter_adjust_reversed.iloc[:, -1].values
+        
+        print("pop_end_for_prop:")
+        print(pop_end_for_prop)
+        print("max_correctables:")
+        print(max_correctables)
+        
+        pop_end_len = len(pop_end_for_prop)
+        for i in np.arange(pop_end_len, step = prop_size):
+            period = [i, min(i + prop_size, pop_end_len)]
+            props_numer = pop_end_for_prop[period[0]:period[1]]
+            props_denom = sum(props_numer)
+            props = props_numer / props_denom
+            print("to_modify_val_total before:")
+            print(to_modify_val_total)
+            print("props:")
+            print(props)
+            props_x_m = np.array(
+                list(map(int, np.round(to_modify_val_total * props)))
+            )
+            print("props_x_m:")
+            print(props_x_m)
+            max_correct_for_each =\
+                max_correctables[period[0]:period[1]]
+            print("max_correct_for_each:")
+            print(max_correct_for_each)
+            min_from_each = np.minimum(props_x_m, max_correct_for_each)
+            print("min_from_each:")
+            print(min_from_each)
+            loc_max_prop = np.where(props == max(props))[0][0]
+            print("loc_max_prop:")
+            print(loc_max_prop)
+            sum_min_from_each = sum(min_from_each)
+            to_modify_val_total -= sum_min_from_each
+            print("to_modify_val_total after:")
+            print(to_modify_val_total)
+            if props_x_m[loc_max_prop] <= max_correct_for_each[loc_max_prop]:
+                more_possible_correction =\
+                    max_correct_for_each[loc_max_prop] -\
+                    props_x_m[loc_max_prop]
+                print("more_possible_correction:")
+                print(more_possible_correction)
+                min_mod_cor = min(
+                    to_modify_val_total,
+                    more_possible_correction
+                )
+                min_from_each[loc_max_prop] += min_mod_cor
+                to_modify_val_total -= min_mod_cor
+                print("min_from_each further:")
+                print(min_from_each)
+                print("to_modify_val_total after further:")
+                print(to_modify_val_total)
+                result.extend(min_from_each)
+            else:
+                result.extend(min_from_each)
+            print("result:")
+            print(result)
+            print(len(result))
+            print("")
+
+    df_comp_counter_adjust_reversed[comp_K] = result
+    del df_comp_counter_adjust_reversed[comp_J]
+    df_comp_counter_adjust_reversed\
+        .sort_values(
+            pop_groups, 
+            ascending = [True, True], 
+            inplace = True
+        )
+    
+    return df_comp_counter_adjust_reversed
